@@ -2,6 +2,7 @@
 use std::{collections::BTreeMap, iter::zip, sync::Arc};
 
 use anyhow::Context;
+use itertools::Itertools;
 use rr_util::{
     name::Name,
     symbolic_size::{SymbolicSizeConstraint, SymbolicSizeConstraints},
@@ -103,9 +104,6 @@ fn compute_named_axes(
     children: &[CircuitRc],
     child_axis_map: &Vec<Vec<Option<usize>>>,
 ) -> NamedAxes {
-    if !children.iter().any(|x| !x.info().named_axes.is_empty()) {
-        return BTreeMap::new();
-    }
     let mut result: NamedAxes = BTreeMap::new();
     for (mp, child) in zip(child_axis_map, children) {
         for (ax, name) in &child.info().named_axes {
@@ -121,27 +119,21 @@ fn compute_free_symbols(children: &[CircuitRc]) -> Option<Arc<IndexSet<Symbol>>>
     // ideally, symbol would just return itself rather than doing this check here
     // but, then the symbol would have a circular reference to itself
     // So we can only include children and then we special case syms in various places. : /
-    children.iter().fold(None, |result, child| {
-        if let Some(result_some) = result {
-            if let Some(sym) = child.as_symbol() {
-                let mut result_mut = (*result_some).clone();
-                result_mut.insert(sym.clone());
-                Some(Arc::new(result_mut))
-            } else if let Some(other) = &child.info().free_symbols {
-                let mut result_mut = (*result_some).clone();
-                result_mut.extend(other.iter().cloned());
-                Some(Arc::new(result_mut))
-            } else {
-                Some(result_some)
-            }
-        } else {
-            if let Some(sym) = child.as_symbol() {
+    children
+        .iter()
+        .filter_map(|x| {
+            if let Some(sym) = x.as_symbol() {
                 Some(Arc::new(IndexSet::from_iter([sym.clone()])))
             } else {
-                child.info().free_symbols.clone()
+                x.info().free_symbols.clone()
             }
-        }
-    })
+        })
+        .at_most_one()
+        .unwrap_or_else(|mut it| {
+            let mut a = (*it.next().unwrap()).clone();
+            it.for_each(|x| a.extend(x.iter().cloned()));
+            Some(Arc::new(a))
+        })
 }
 
 pub trait CircuitNodeSetNonHashInfo: CircuitNodePrivate {
@@ -162,6 +154,12 @@ impl<T: CircuitNodeComputeInfoImpl + CircuitNodePrivate> CircuitNodeSetNonHashIn
         )?;
         self.info_mut().named_axes = if let Some(out) = self.already_set_named_axes() {
             out
+        } else if self
+            .children_sl()
+            .iter()
+            .all(|x| x.info().named_axes.is_empty())
+        {
+            BTreeMap::new()
         } else {
             compute_named_axes(self.children_sl(), &self.child_axis_map())
         };
