@@ -1,7 +1,6 @@
 use std::{cmp::Ordering, iter::zip};
 
 use anyhow::{anyhow, bail, Context, Result};
-use itertools::Itertools;
 use macro_rules_attribute::apply;
 use num_bigint::BigUint;
 use pyo3::{prelude::*, types::PyTuple};
@@ -23,9 +22,7 @@ use rr_util::{
     pycall,
     rearrange_spec::{OpSize, RearrangeSpec}, // TODO: use new rearrange spec
     sv,
-    tensor_util::{
-        broadcast_shapes, Slice, TensorAxisIndex, TensorIndex, TorchDeviceDtype, TorchDeviceDtypeOp,
-    },
+    tensor_util::{broadcast_shapes, Slice, TensorAxisIndex, TensorIndex, TorchDeviceDtypeOp},
     util::{
         cumsum, dict_to_list, {hashmap_collect_except_duplicates, AxisInt},
     },
@@ -44,7 +41,7 @@ use crate::{
     },
     new_rc_unwrap,
     prelude::*,
-    upcast_tensors_for_eval, CachedCircuitInfo, CircuitNodeSelfOnlyHash, ConstructError,
+    upcast_tensor_device_dtypes, CachedCircuitInfo, CircuitNodeSelfOnlyHash, ConstructError,
     PyCircuitBase, Scalar, Shape,
 };
 
@@ -122,12 +119,12 @@ impl CircuitNode for Einsum {
     }
 
     fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
-        let tensors = upcast_tensors_for_eval!(self, tensors);
+        let tensors = upcast_tensor_device_dtypes(tensors);
         Python::with_gil(|py| {
             if self.in_axes.is_empty() {
                 return Ok(scalar_to_tensor_py(py, 1., sv![], Default::default())?);
             }
-            let out_axes_deduped: EinsumAxes = self.out_axes.iter().unique().cloned().collect();
+            let out_axes_deduped: EinsumAxes = self.out_axes.unique();
             let result_non_diag: Tensor = einsum_py(
                 py,
                 zip(tensors.iter().cloned(), self.in_axes.iter().cloned()).collect(),
@@ -368,13 +365,12 @@ impl Einsum {
 
     #[staticmethod]
     pub fn new_diag(node: CircuitRc, ints: EinsumAxes, name: Option<Name>) -> Self {
-        let deduped = ints.iter().unique().cloned().collect();
-        Einsum::try_new(vec![(node, deduped)], ints, name).unwrap()
+        Einsum::try_new(vec![(node, ints.unique())], ints, name).unwrap()
     }
 
     #[staticmethod]
     pub fn new_trace(node: CircuitRc, ints: EinsumAxes, name: Option<Name>) -> Self {
-        let deduped = ints.iter().unique().cloned().collect();
+        let deduped = ints.unique();
         Einsum::try_new(vec![(node, ints)], deduped, name).unwrap()
     }
 
@@ -670,7 +666,7 @@ impl CircuitNode for Add {
         if tensors.is_empty() {
             return Ok(scalar_to_tensor(0.0, sv!(), Default::default())?);
         }
-        let tensors = upcast_tensors_for_eval!(self, tensors);
+        let tensors = upcast_tensor_device_dtypes(tensors);
         let mut out = tensors[0].clone();
         Python::with_gil(|py| {
             for tensor in tensors[1..].iter() {
@@ -940,8 +936,7 @@ impl CircuitNodeComputeInfoImpl for Index {
                 .iter()
                 .filter_map(|x| match x {
                     TensorAxisIndex::Tensor(tensor) if !tensor.shape().is_empty() => {
-                        let mut out: TorchDeviceDtypeOp =
-                            TorchDeviceDtype::from_tensor(tensor).into();
+                        let mut out: TorchDeviceDtypeOp = tensor.device_dtype().into();
                         out.dtype = None;
                         Some(out)
                     }
@@ -984,7 +979,7 @@ impl CircuitNode for Index {
 
     fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
         assert_eq!(tensors.len(), 1);
-        let tensors = upcast_tensors_for_eval!(self, tensors);
+        let tensors = upcast_tensor_device_dtypes(tensors);
         Python::with_gil(|py| {
             PY_CIRCUIT_ITEMS
                 .circ_compiler_util
@@ -1111,7 +1106,7 @@ impl Index {
                         ))),
                         match &tensors_index.0[0] {
                             TensorAxisIndex::Tensor(t) => {
-                                TorchDeviceDtype::from_tensor(t).device
+                                t.device_dtype().device
                             }
                             _ => panic!(),
                         }
@@ -1293,7 +1288,7 @@ impl CircuitNode for Concat {
     }
 
     fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
-        let tensors = upcast_tensors_for_eval!(self, tensors);
+        let tensors = upcast_tensor_device_dtypes(tensors);
         Python::with_gil(|py| {
             PY_UTILS
                 .torch
@@ -1703,7 +1698,7 @@ impl CircuitNode for Conv {
     }
 
     fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
-        let tensors = upcast_tensors_for_eval!(self, tensors);
+        let tensors = upcast_tensor_device_dtypes(tensors);
         pycall!(
             PY_UTILS.conv,
             (
