@@ -120,17 +120,13 @@ pub fn expand_node(
         usize,
     ) -> Result<CircuitRc>,
 ) -> Result<CircuitRc> {
-    if inputs.len() != circuit.children().count() {
+    if inputs.len() != circuit.num_children() {
         bail!(ExpandError::WrongNumChildren {
-            expected: circuit.children().count(),
+            expected: circuit.num_children(),
             got: inputs.len(),
         });
     }
-    if inputs
-        .iter()
-        .zip(circuit.children())
-        .all(|(new, old)| new == &old)
-    {
+    if inputs == circuit.children_sl() {
         return Ok(circuit); // in all identical case we can return same circuit for efficiency
     }
     if inputs
@@ -202,7 +198,7 @@ pub fn expand_node(
             .unwrap_or(sizes[0])
     };
 
-    let out = match &**circuit {
+    let out: CircuitRc = match &**circuit {
         Circuit::SetSymbolicShape(s) => {
             let child_shape = inputs[0].shape();
             let s2: Shape = s
@@ -223,20 +219,17 @@ pub fn expand_node(
                 .concat()
                 .to_smallvec();
             let c = inputs[0].clone();
-            Ok(if s2 != *child_shape {
+            if s2 != *child_shape {
                 SetSymbolicShape::nrc(c, s2, s.info().name)
             } else {
                 c
-            })
+            }
         }
-        Circuit::Array(_) | Circuit::Symbol(_) | Circuit::Scalar(_) => Ok(circuit.clone()),
+        Circuit::Array(_) | Circuit::Symbol(_) | Circuit::Scalar(_) => circuit.clone(),
         // handling of rearrange is basically a hack to deal with simp (and maybe other places?) not preserving wildcards
         // the correct thing would be to preserve wildcards and only allow expand changing dims where there's wildcards in rearrange
         Circuit::Rearrange(node) => {
-            let input_shape_non_batch = inputs[0].info().shape[batch_ranks[0]..]
-                .iter()
-                .cloned()
-                .collect();
+            let input_shape_non_batch = inputs[0].info().shape[batch_ranks[0]..].to_smallvec();
 
             // TODO: test me better!
             let (new_input, expanded_spec) = match node
@@ -269,7 +262,7 @@ pub fn expand_node(
             };
 
             let new_spec = expanded_spec.add_batch_dims(batch_ranks[0]);
-            Rearrange::try_new(new_input, new_spec, circuit.info().name).map(|x| x.rc())
+            Rearrange::try_new(new_input, new_spec, circuit.info().name)?.rc()
         }
         // I think we just don't do any symbolic shape munging in Index/Scatter?
         Circuit::Index(node) => {
@@ -279,13 +272,13 @@ pub fn expand_node(
                     && node.index.0[i] != TensorAxisIndex::IDENT
                 {
                     bail!(ExpandError::FixedIndex {
-                        index: node.index.clone(),
+                        index: circuit.clone(),
                         old_shape: node.node().info().shape.clone(),
                         new_shape: inputs[0].info().shape.clone(),
                     });
                 }
             }
-            Ok(Index::nrc(
+            Index::nrc(
                 inputs[0].clone(),
                 TensorIndex(
                     vec![TensorAxisIndex::IDENT; batch_ranks[0]]
@@ -294,7 +287,7 @@ pub fn expand_node(
                         .collect(),
                 ),
                 node.info().name,
-            ))
+            )
         }
         Circuit::Scatter(node) => {
             // for now non-batch non-identity dims can't change
@@ -303,13 +296,13 @@ pub fn expand_node(
                     && node.index.0[i] != TensorAxisIndex::IDENT
                 {
                     bail!(ExpandError::FixedIndex {
-                        index: node.index.clone(),
+                        index: circuit.clone(),
                         old_shape: node.node().info().shape.clone(),
                         new_shape: inputs[0].info().shape.clone(),
                     });
                 }
             }
-            Ok(Scatter::nrc(
+            Scatter::nrc(
                 inputs[0].clone(),
                 TensorIndex(
                     vec![TensorAxisIndex::IDENT; batch_ranks[0]]
@@ -323,7 +316,7 @@ pub fn expand_node(
                     .chain(node.info().shape.iter().cloned())
                     .collect(),
                 node.info().name,
-            ))
+            )
         }
         Circuit::Concat(node) => {
             // TODO: error on concat axis having symbolic size? (or maybe concat axis is fine???
@@ -378,7 +371,7 @@ pub fn expand_node(
                 inputs.clone()
             };
 
-            Concat::try_new(inputs, new_axis, node.info().name).map(|x| x.rc())
+            Concat::try_new(inputs, new_axis, node.info().name)?.rc()
         }
         Circuit::Add(node) => {
             let inputs: Vec<_> = inputs
@@ -426,7 +419,7 @@ pub fn expand_node(
                     .map(|inp| {
                         SetSymbolicShape::some_set_and_symbolic_neq(
                             inp.clone(),
-                            full_shape[max - inp.ndim()..].iter().cloned().collect(),
+                            full_shape[max - inp.ndim()..].to_smallvec(),
                             None,
                         )
                     })
@@ -438,7 +431,7 @@ pub fn expand_node(
                 inputs.clone()
             };
 
-            Add::try_new(inputs, node.info().name).map(|x| x.rc())
+            Add::try_new(inputs, node.info().name)?.rc()
         }
         Circuit::GeneralFunction(node) => {
             // TODO: symbolic? (annoying to handle, would require changes to spec)
@@ -459,8 +452,8 @@ pub fn expand_node(
                 })?,
                 node.spec.clone(),
                 node.info().name,
-            )
-            .map(|x| x.rc())
+            )?
+            .rc()
         }
         Circuit::Einsum(node) => {
             let br = batch_rank;
@@ -528,9 +521,9 @@ pub fn expand_node(
                     )
                 })?;
 
-            Einsum::try_new(new_args, out_axes, node.info().name).map(|x| x.rc())
+            Einsum::try_new(new_args, out_axes, node.info().name)?.rc()
         }
-        Circuit::Tag(node) => Ok(Tag::nrc(inputs[0].clone(), node.uuid, node.info().name)),
+        Circuit::Tag(node) => Tag::nrc(inputs[0].clone(), node.uuid, node.info().name),
         Circuit::Cumulant(node) => {
             // For now, only supports expansion in easy cases
             if zip(inputs, node.children())
@@ -539,7 +532,7 @@ pub fn expand_node(
             {
                 bail!(ExpandError::CumulantRankChanged { circuit })
             }
-            Ok(Cumulant::nrc(inputs.clone(), node.info().name))
+            Cumulant::nrc(inputs.clone(), node.info().name)
         }
         Circuit::Module(node) => {
             assert_eq!(inputs.len(), node.num_children());
@@ -800,18 +793,18 @@ pub fn expand_node(
                 out
             };
 
-            Ok(final_out)
+            final_out
         }
         _ => {
-            if inputs[..] == circuit.children().collect::<Vec<_>>()[..] {
-                Ok(circuit.clone())
+            if &inputs[..] == circuit.children_sl() {
+                circuit.clone()
             } else {
                 bail!(ExpandError::NodeUnhandledVariant {
                     variant: circuit.variant_string(),
                 })
             }
         }
-    }?;
+    };
     assert!(
         out.ndim() == circuit.ndim() + batch_rank,
         "batching assumption violated!"
@@ -835,7 +828,7 @@ pub enum ExpandError {
 
     #[error("Trying to expand fixed index, index {index:?} old shape{old_shape:?} new shape {new_shape:?} ({e_name})")]
     FixedIndex {
-        index: TensorIndex,
+        index: CircuitRc,
         old_shape: Shape,
         new_shape: Shape,
     },
