@@ -13,11 +13,11 @@ use rand::Rng;
 use regex::Regex;
 use rr_util::{
     name::Name,
-    py_types::{assert_tensors_close, ExtraPySelfOps, PyShape, Tensor, PY_UTILS},
+    py_types::{assert_tensors_close, ExtraPySelfOps, PyShape, Tensor, PY_UTILS, SELF_MODULE},
     pycall, python_error_exception, simple_from,
     tensor_util::{
-        broadcast_shapes_impl, check_canon_idxs, MiscInputError, Shape, TorchDevice,
-        TorchDeviceDtypeOp, TorchDtype,
+        broadcast_shapes_impl, check_canon_idxs, eq_or_symbolic, MiscInputError, Shape,
+        TorchDevice, TorchDeviceDtypeOp, TorchDtype,
     },
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
@@ -30,7 +30,8 @@ use crate::{
     circuit_utils::OperatorPriority,
     new_rc_unwrap,
     prelude::*,
-    Array, CachedCircuitInfo, HashBytes, PyCircuitBase,
+    upcast_tensor_device_dtypes, upcast_tensor_devices, Array, CachedCircuitInfo, HashBytes,
+    PyCircuitBase,
 };
 
 macro_rules! gf_gen {
@@ -151,7 +152,7 @@ impl GeneralFunctionShapeInfo {
         PyShape(self.shape.clone())
     }
 
-    /// no checking done here, we assume valid
+    /// no checking done here, validation done in validate
     #[new]
     pub fn new(
         shape: Shape,
@@ -166,6 +167,36 @@ impl GeneralFunctionShapeInfo {
     }
 }
 
+impl GeneralFunctionShapeInfo {
+    pub fn validate(&self, shapes: &[Shape]) -> Result<()> {
+        if shapes.len() != self.input_batchability.len() {
+            bail!(GeneralFunctionShapeError::WrongNumShapes {
+                got: shapes.len(),
+                expected: self.input_batchability.len()
+            })
+        }
+
+        if self.input_batchability.iter().any(|b| *b) {
+            let batch_dims = self.shape.len() - self.num_non_batchable_output_dims as usize;
+            let out_batch = &self.shape[..batch_dims];
+            let batches: Vec<&[usize]> = shapes
+                .iter()
+                .zip(&self.input_batchability)
+                .filter(|(_, b)| **b)
+                .map(|(s, _)| &s[..batch_dims])
+                .collect();
+            if !batches.iter().all(|x| eq_or_symbolic(x, &out_batch)) {
+                bail!(GeneralFunctionShapeError::BatchDimsNotEqual {
+                    batches: batches.into_iter().map(|x| x.into()).collect(),
+                    output_batch: out_batch.into()
+                })
+            }
+        }
+
+        Ok(())
+    }
+}
+
 #[pyclass]
 #[derive(Debug, Clone)]
 pub struct GeneralFunctionSimpleSpec {
@@ -219,6 +250,7 @@ impl SpecTrait for GeneralFunctionSimpleSpec {
     }
 
     fn function(&self, tensors: &[Tensor]) -> Result<Tensor> {
+        let tensors = upcast_tensor_device_dtypes(tensors);
         Python::with_gil(|py| {
             Ok(self
                 .get_function()
@@ -325,6 +357,7 @@ impl SpecTrait for GeneralFunctionIndexSpec {
     }
 
     fn function(&self, tensors: &[Tensor]) -> Result<Tensor> {
+        let tensors = upcast_tensor_devices(tensors);
         Python::with_gil(|py| {
             Ok(PY_UTILS
                 .gen_index_function
@@ -858,6 +891,7 @@ impl SpecTrait for GeneralFunctionMultinomialSpec {
     }
 
     fn function(&self, tensors: &[Tensor]) -> Result<Tensor> {
+        let tensors = upcast_tensor_devices(tensors);
         Ok(sample_multinomial(
             tensors[0].clone(),
             self.shape.clone(),
@@ -1045,22 +1079,8 @@ impl fmt::Debug for PyWrap {
 
 simple_from!(|x: PyWrap| -> GeneralFunctionSpec { GeneralFunctionSpec::Py(Box::new(x)) });
 
-pub static PY_WRAP_BASE: GILLazyPy<PyObject> = GILLazyPy::new_py(|py| {
-    let py_part = PyModule::from_code(
-        py,
-        include_str!(concat!(
-            env!("CARGO_MANIFEST_DIR"),
-            "/py_general_function_spec.py"
-        )),
-        concat!(env!("CARGO_MANIFEST_DIR"), "/py_general_function_spec.py"),
-        "py_general_function_spec",
-    )
-    .unwrap();
-
-    let get = |s: &str| py_part.getattr(s).unwrap().into();
-
-    get("GeneralFunctionSpecBase")
-});
+pub static PY_WRAP_BASE: GILLazyPy<PyObject> =
+    GILLazyPy::new_py(|py| SELF_MODULE.getattr(py, "GeneralFunctionSpecBase").unwrap());
 
 impl<'source> pyo3::FromPyObject<'source> for Box<PyWrap> {
     fn extract(ob: &'source PyAny) -> PyResult<Self> {
@@ -1073,12 +1093,6 @@ impl<'source> pyo3::FromPyObject<'source> for Box<PyWrap> {
             return Err(err.into());
         }
 
-        hasher.update(uuid!("de3124ee-154c-4da8-bdb3-b7496ae6223c").as_bytes()); // uuid for PyWrap
-        let bytes: Vec<u8> = Python::with_gil(|py| -> Result<_> {
-            Ok(ob.call_method0(py, "compute_hash_bytes")?.extract(py)?)
-        })?;
-        hasher.update(&bytes);
-
         let name: String =
             Python::with_gil(|py| -> Result<_> { Ok(ob.getattr(py, "name")?.extract(py)?) })?;
 
@@ -1093,6 +1107,16 @@ impl<'source> pyo3::FromPyObject<'source> for Box<PyWrap> {
             PyClassLocator::parse(p)?;
         }
 
+        hasher.update(uuid!("de3124ee-154c-4da8-bdb3-b7496ae6223c").as_bytes()); // uuid for PyWrap
+        hasher.update(name.as_bytes());
+        hasher.update(uuid!("dd4e76ed-2d6f-4a11-b5e6-5e32399dbe90").as_bytes());
+        hasher.update(path.as_ref().map_or(&[], |p| p.as_bytes()));
+        hasher.update(uuid!("a9737433-7947-46d7-a509-062e99daaebe").as_bytes());
+        let bytes: Vec<u8> = Python::with_gil(|py| -> Result<_> {
+            Ok(ob.call_method0(py, "compute_hash_bytes")?.extract(py)?)
+        })?;
+        hasher.update(&bytes);
+
         Ok(Box::new(PyWrap {
             name: name.into(),
             path,
@@ -1269,11 +1293,7 @@ impl GeneralFunction {
             .map(|x| x.shape().clone())
             .collect::<Vec<Shape>>();
 
-        let GeneralFunctionShapeInfo {
-            shape,
-            num_non_batchable_output_dims,
-            input_batchability,
-        } = spec.get_shape_info(&shapes).with_context(|| {
+        let si = spec.get_shape_info(&shapes).with_context(|| {
             format!(
                 "failed to compute shape info in new with spec={:?} input_shapes={:?}",
                 spec,
@@ -1284,6 +1304,23 @@ impl GeneralFunction {
             )
         })?;
 
+        si.validate(&shapes).with_context(|| {
+            format!(
+                "invalid shape info in new with spec={:?} input_shapes={:?}",
+                spec,
+                nodes
+                    .iter()
+                    .map(|x| x.shape().clone())
+                    .collect::<Vec<Shape>>()
+            )
+        })?;
+
+        let GeneralFunctionShapeInfo {
+            shape,
+            num_non_batchable_output_dims,
+            input_batchability,
+        } = si;
+
         let out = Self {
             spec,
             info: CachedCircuitInfo::incomplete(name, shape, nodes),
@@ -2160,6 +2197,14 @@ pub enum GeneralFunctionShapeError {
         removed_from_end: u8,
     },
 
+    #[error(
+        "batch dims not equal for all inputs: batches={batches:?} output_batch={output_batch:?} ({e_name})"
+    )]
+    BatchDimsNotEqual {
+        batches: Vec<Shape>,
+        output_batch: Shape,
+    },
+
     #[error("x_shape={x_shape:?} index_shape={index_shape:?} batch_x={batch_x} ({e_name})")]
     IndexShapeInvalid {
         x_shape: Shape,
