@@ -452,7 +452,7 @@ impl Parser {
                     partial_circuits.insert(
                         circuit_ident.clone(),
                         PartialCirc {
-                            name: name,
+                            name,
                             shape,
                             variant: variant.ok_or_else(|| {
                                 ParseCircuitError::RegexDidntMatchGroup {
