@@ -8,7 +8,7 @@ pub struct SetOfCircuitIdentities(HashSet<HashBytes>); // could use something th
 #[pymethods]
 impl SetOfCircuitIdentities {
     #[new]
-    #[args(hs = "Default::default()")]
+    #[pyo3(signature=(hs = Default::default()))]
     pub fn new(hs: HashSet<HashBytes>) -> Self {
         Self(hs)
     }
