@@ -87,7 +87,7 @@ pub fn add_elim_zeros(add: &Add) -> Option<Add> {
         .children()
         .filter_map(|node| match &**node {
             Circuit::Scalar(scalar) => {
-                if scalar.value == 0.0 && scalar.info().rank() == 0 {
+                if scalar.value == 0.0 {
                     did_anything = true;
                     None
                 } else {
@@ -100,7 +100,12 @@ pub fn add_elim_zeros(add: &Add) -> Option<Add> {
     if !did_anything {
         None
     } else {
-        Some(Add::new(new_operands, add.info().name))
+        let out = Add::new(new_operands, add.info().name);
+        if out.shape() == add.shape() {
+            Some(out)
+        } else {
+            None
+        } // in case the scalars caused broadcasting
     }
 }
 
@@ -496,7 +501,11 @@ pub fn index_fuse(index: &Index) -> Option<Index> {
     index.node().as_index().and_then(|inner| {
         Some(Index::new(
             inner.node().clone(),
-            compose(&index.index, &inner.index)?,
+            compose(
+                &index.index,
+                &inner.index.canonicalize(inner.node().shape()),
+            )
+            .unwrap(),
             index.info().name.map(|n| {
                 if n.ends_with(" idx idx") {
                     n.strip_suffix(" idx").unwrap().into()
@@ -990,7 +999,7 @@ pub fn distribute_once_op(einsum: &Einsum, operand_idx: usize, do_broadcasts: bo
 #[pyfunction]
 pub fn distribute_all(einsum: &Einsum) -> Option<Add> {
     let add_op_idxs = filter_to_idx(einsum.args(), |(node, _ints)| {
-        matches!(&***node, Circuit::Add(_)) && node.children().count() > 0
+        matches!(&***node, Circuit::Add(_)) && node.num_children() > 0
     });
     if add_op_idxs.is_empty() {
         return None;
@@ -1219,9 +1228,10 @@ pub fn axis_index_split_sections_for_concat(
     sections: &Vec<usize>,
 ) -> Result<Vec<Option<TensorAxisIndex>>> {
     let starts = cumsum(sections);
+    let len = sections.iter().sum();
     let out = match axis_index {
         TensorAxisIndex::Single(single) => {
-            let single = check_canon_idxs(sections.iter().sum(), &[*single])
+            let single = check_canon_idxs(len, &[*single])
                 .context("out of bounds when checking index for concat push down index")?[0];
             let outer_idx = starts.iter().take_while(|start| **start <= single).count() - 1;
             let inner_idx = single - starts[outer_idx];
@@ -1230,17 +1240,35 @@ pub fn axis_index_split_sections_for_concat(
             result
         }
         &TensorAxisIndex::Slice(slice) => {
-            let uslice: USlice = slice.to_uslice(sections.iter().sum());
+            let uslice: USlice = slice.to_uslice(len);
             let mut result = vec![None; sections.len()];
             for (i, (start, sec_len)) in zip(starts.iter(), sections.iter()).enumerate() {
                 let stop = start + sec_len;
-                if *start < uslice.stop && stop > uslice.start {
+                if *start < uslice.stop
+                    && stop > uslice.start
+                    && ((*start != stop) || (*start < uslice.stop - 1))
+                {
                     result[i] = Some(TensorAxisIndex::new_plain_slice(
                         uslice.start.saturating_sub(*start),
                         std::cmp::min(*sec_len, uslice.stop - start),
                     ))
                 }
             }
+            if uslice.start == uslice.stop && result.iter().all(|x| x.is_none()) {
+                if uslice.start == 0 {
+                    result[0] = Some(TensorAxisIndex::Slice(uslice.into()))
+                } else if uslice.stop == len {
+                    let start = uslice.start.saturating_sub(*starts.last().unwrap());
+                    *result.last_mut().unwrap() =
+                        Some(TensorAxisIndex::new_plain_slice(start, start))
+                } else {
+                    // at boundary between concattands
+                    bail!(PushDownIndexError::ZeroLengthIndexAmbiguous {})
+                }
+            }
+            if sections.len() == 1 && result[0] == None {
+                result[0] = Some(TensorAxisIndex::Slice(uslice.into()))
+            }
             result
         }
         TensorAxisIndex::Tensor(tensor) => {
@@ -1255,9 +1283,15 @@ pub fn axis_index_split_sections_for_concat(
 pub fn push_down_index_raw(
     node: &Index,
     allow_partial_pushdown: bool,
-    mut call_on_sub: impl FnMut(usize, Index) -> Result<CircuitRc>,
+    call_on_sub: &mut dyn FnMut(usize, Index) -> Result<CircuitRc>,
     suffix: Option<String>,
 ) -> Result<CircuitRc> {
+    let out_name = node.node().info().name.map(|n_s| {
+        suffix
+            .as_ref()
+            .map(|s| format!("{n_s}{s}").into())
+            .unwrap_or(n_s)
+    });
     let mut on_sub = |child_idx,
                       node: CircuitRc,
                       index,
@@ -1266,22 +1300,17 @@ pub fn push_down_index_raw(
         let new_name: Option<Name> = force_name.unwrap_or_else(|| {
             suffix
                 .as_ref()
-                .map(|s| node.info().name.map(|n_s| format!("{n_s}{s}").into()))
-                .flatten()
+                .and_then(|s| node.info().name.map(|n_s| format!("{n_s}_idx_{s}").into()))
         });
         call_on_sub(child_idx, Index::new(node, index, new_name))
     };
-    let out_name = node.info().name;
-    let partial_name = || {
-        suffix
-            .as_ref()
-            .map(|s| {
-                node.node()
-                    .info()
-                    .name
-                    .map(|inner_name| format!("{inner_name}_partial_{s}").into())
-            })
-            .flatten()
+    let idx_partial_name = || {
+        suffix.as_ref().and_then(|s| {
+            node.node()
+                .info()
+                .name
+                .map(|inner_name| format!("{inner_name}_partial_{s}").into())
+        })
     };
 
     let handle_non_batchable_remaining = |rank_to_pass, new_circ: CircuitRc| {
@@ -1304,11 +1333,7 @@ pub fn push_down_index_raw(
                     .chain(top_axis_indices)
                     .collect(),
             );
-            Ok(Index::nrc(
-                new_circ.rename(partial_name()),
-                index_top,
-                out_name.clone(),
-            ))
+            Ok(Index::nrc(new_circ, index_top, idx_partial_name()))
         } else {
             Err((passed_rank_now, top_axis_indices))
         }
@@ -1430,14 +1455,14 @@ pub fn push_down_index_raw(
                         .map(|(i, operand)| on_sub(i, operand, index_non_axis.clone(), None))
                         .collect::<Result<_>>()?,
                     new_axis,
-                    partial_name(),
+                    out_name,
                 );
                 let final_index = TensorIndex::new_single(
                     node.index.0[inner.axis].clone(),
                     new_axis,
                     new_concat.info().rank(),
                 );
-                Index::nrc(new_concat, final_index, out_name)
+                Index::nrc(new_concat, final_index, idx_partial_name())
             }
         }
         Circuit::GeneralFunction(inner) => {
@@ -1564,7 +1589,7 @@ pub fn push_down_index_raw(
             if out_index.is_identity(new_einsum.shape()) {
                 new_einsum
             } else if allow_partial_pushdown {
-                Index::nrc(new_einsum.rename(partial_name()), out_index, out_name)
+                Index::nrc(new_einsum, out_index, idx_partial_name())
             } else {
                 bail!(PushDownIndexError::EinsumSomeAxesNotPossible {
                     index_node: node.clone(),
@@ -1639,7 +1664,7 @@ pub fn push_down_index_raw(
                 inner_node,
                 TensorIndex(new_scatter.iter().cloned().map(USlice::into).collect()),
                 new_scatter_shape,
-                out_name.clone(),
+                out_name,
             );
 
             let out_index = TensorIndex(new_outer);
@@ -1647,7 +1672,7 @@ pub fn push_down_index_raw(
             if out_index.is_identity(new_scatter.shape()) {
                 new_scatter
             } else if allow_partial_pushdown {
-                Index::nrc(new_scatter.rename(partial_name()), out_index, out_name)
+                Index::nrc(new_scatter, out_index, idx_partial_name())
             } else {
                 bail!(PushDownIndexError::ScatterSomeAxesNotPossible {
                     index_node: node.clone(),
@@ -1735,13 +1760,13 @@ pub fn push_down_index_once(
     push_down_index_raw(
         &node,
         allow_partial_pushdown.unwrap_or(false),
-        |_, x| Ok(x.rc()),
+        &mut |_, x| Ok(x.rc()),
         suffix,
     )
 }
 
 pub fn push_down_index_op(node: &Index) -> Option<CircuitRc> {
-    push_down_index_raw(node, true, |_, x| Ok(x.rc()), None).ok()
+    push_down_index_raw(node, true, &mut |_, x| Ok(x.rc()), None).ok()
 }
 
 #[pyfunction]
@@ -2096,6 +2121,9 @@ pub enum PushDownIndexError {
 
     #[error("tensors aren't currently supported tensor={tensor:?} ({e_name})")]
     ConcatSplitSectionsTensorUnsupported { tensor: IndexTensor },
+
+    #[error("zero-length index and no unique choice ({e_name})")]
+    ZeroLengthIndexAmbiguous {},
 }
 
 #[apply(python_error_exception)]
