@@ -214,8 +214,7 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> Resul
     no_memory_limit_context.settings.max_single_tensor_memory = usize::MAX;
     // use named axes to record the ints used in intermediate einsums
     let einsum_args_named_axes: Vec<(CircuitRc, EinsumAxes)> = einsum
-        .args
-        .iter()
+        .args()
         .enumerate()
         .map(|(i, (c, ints))| {
             (
@@ -316,7 +315,7 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> Resul
             .iter()
             .position(|(s, _ints)| sym.uuid==s.as_symbol().unwrap().uuid)
         {
-            return Some(einsum.args[p].0.clone());
+            return Some(einsum.children_sl()[p].clone());
         }
         None
     })
