@@ -24,26 +24,26 @@ pub fn canonicalize_node_py(circuit: CircuitRc) -> CircuitRc {
 pub fn canonicalize_node_op(circuit: CircuitRc) -> Option<CircuitRc> {
     match &**circuit {
         Circuit::Rearrange(rearrange) => Some(Rearrange::nrc(
-            rearrange.node.clone(),
+            rearrange.node().clone(),
             rearrange
                 .spec
-                .conform_to_input_shape(&rearrange.node.info().shape)
+                .conform_to_input_shape(&rearrange.node().info().shape)
                 .unwrap()
                 .canonicalize(true),
             circuit.info().name,
         )),
         Circuit::Index(index) => Some(Index::nrc(
-            index.node.clone(),
-            index.index.canonicalize(&index.node.info().shape),
+            index.node().clone(),
+            index.index.canonicalize(&index.node().info().shape),
             index.info().name,
         )),
         Circuit::Add(add) => {
-            let mut nodes_sorted = add.nodes.clone();
+            let mut nodes_sorted = add.children_sl().to_vec();
             nodes_sorted.sort_by_key(numel_sort_key);
             Some(Add::nrc(nodes_sorted, add.info().name))
         }
         Circuit::Einsum(einsum) => {
-            let mut args_sorted = einsum.args.clone();
+            let mut args_sorted = einsum.args_cloned();
             args_sorted.sort_by_key(|(node, _ints)| numel_sort_key(node));
             Some(
                 Einsum::try_new(args_sorted, einsum.out_axes.clone(), einsum.info().name)
@@ -81,7 +81,7 @@ pub fn normalize_node_py(circuit: CircuitRc) -> CircuitRc {
 pub fn normalize_node_op(circuit: CircuitRc) -> Option<CircuitRc> {
     match &**circuit {
         Circuit::Rearrange(rearrange) => Some(Rearrange::nrc(
-            rearrange.node.clone(),
+            rearrange.node().clone(),
             rearrange.spec.canonicalize(false),
             circuit.info().name,
         )),
