@@ -74,15 +74,18 @@ pub fn opt_eval_each_subcircuit_until_fail(circuit: CircuitRc, settings: Optimiz
     for circ in topo {
         if circ.info().numel() < max_numel {
             let reference = evaluate(circ.clone()).unwrap();
-            let optimized = optimize_and_evaluate(circ.clone(), settings);
+            let optimized = optimize_and_evaluate(circ.clone(), settings.clone());
             if let Ok(optimized) = optimized {
                 let close_result = is_close(reference, optimized);
                 if close_result.is_err() {
                     println!("{:?}", close_result);
                     circ.printu();
-                    optimize_circuit(circ, &mut OptimizationContext::new_settings(settings))
-                        .unwrap()
-                        .printu();
+                    optimize_circuit(
+                        circ,
+                        &mut OptimizationContext::new_settings(settings.clone()),
+                    )
+                    .unwrap()
+                    .printu();
                 }
             }
         }
