@@ -11,8 +11,11 @@ use rr_util::{
 };
 #[pyfunction]
 pub fn generalfunction_merge_inverses(circuit: &GeneralFunction) -> Option<CircuitRc> {
-    if circuit.nodes.len()==1 && circuit.spec.is_official() && let Circuit::GeneralFunction(inner)=&**circuit.nodes[0] && inner.spec.is_official() && OFFICIAL_GENERALFUNCTION_INVERSES.iter().any(|(a,b)|a==&circuit.spec.name()&&b==&inner.spec.name()) {
-        return Some(inner.nodes[0].clone())
+    if circuit.num_children()==1 && circuit.spec.is_official()
+        && let Circuit::GeneralFunction(inner) = &**circuit.info().children[0]
+        && inner.spec.is_official()
+        && OFFICIAL_GENERALFUNCTION_INVERSES.iter().any(|(a,b)|a==&circuit.spec.name()&&b==&inner.spec.name()) {
+        return Some(inner.info().children[0].clone())
     }
     None
 }
@@ -23,9 +26,11 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
         let name_str: &str = &circuit.spec.name();
         match name_str {
             "softmax" => {
-                if let Circuit::Scalar(_sc) = &**circuit.nodes[0] {
+                if let Circuit::Scalar(_sc) = &**circuit.children_sl()[0] {
                     let scalar: f64 = 1.0
-                        / circuit.nodes[0].info().shape[circuit.nodes[0].info().rank() - 1] as f64;
+                        / circuit.children_sl()[0].info().shape
+                            [circuit.children_sl()[0].info().rank() - 1]
+                            as f64;
                     return Some(Scalar::nrc(
                         scalar,
                         circuit.info().shape.clone(),
@@ -34,9 +39,11 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
                 }
             }
             "log_softmax" => {
-                if let Circuit::Scalar(_sc) = &**circuit.nodes[0] {
+                if let Circuit::Scalar(_sc) = &**circuit.children_sl()[0] {
                     let scalar: f64 = (1.0
-                        / circuit.nodes[0].info().shape[circuit.nodes[0].info().rank() - 1] as f64)
+                        / circuit.children_sl()[0].info().shape
+                            [circuit.children_sl()[0].info().rank() - 1]
+                            as f64)
                         .ln();
                     return Some(Scalar::nrc(
                         scalar,
@@ -46,12 +53,13 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
                 }
             }
             "last_dim_size" => {
-                let last_dim_maybe_symbolic =
-                    circuit.nodes[0].info().shape[circuit.nodes[0].info().rank() - 1];
+                let last_dim_maybe_symbolic = circuit.children_sl()[0].info().shape
+                    [circuit.children_sl()[0].info().rank() - 1];
                 if !SymbolicSizeProduct::has_symbolic(last_dim_maybe_symbolic) {
                     return Some(Scalar::nrc(
                         last_dim_maybe_symbolic as f64,
-                        circuit.nodes[0].info().shape[..circuit.nodes[0].info().rank() - 1]
+                        circuit.children_sl()[0].info().shape
+                            [..circuit.children_sl()[0].info().rank() - 1]
                             .iter()
                             .cloned()
                             .collect(),
@@ -70,10 +78,10 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
 pub fn generalfunction_evaluate_simple(node: &GeneralFunction) -> Option<CircuitRc> {
     if node.num_non_batchable_output_dims == 0
         && node.info().rank() == 0
-        && node.nodes.len() == 1
-        && node.nodes[0].info().rank() == 0
+        && node.num_children() == 1
+        && node.children_sl()[0].info().rank() == 0
     {
-        return node.nodes[0].as_scalar().map(|_inner| {
+        return node.children_sl()[0].as_scalar().map(|_inner| {
             Python::with_gil(|py| {
                 Scalar::new(
                     evaluate(node.crc())
@@ -98,10 +106,10 @@ pub fn generalfunction_evaluate_simple(node: &GeneralFunction) -> Option<Circuit
 #[pyfunction]
 pub fn generalfunction_gen_index_const_to_index(node: &GeneralFunction) -> Option<CircuitRc> {
     if let GeneralFunctionSpec::Index(i) = &node.spec
-        && let Circuit::Array(a) = &**node.nodes[1]
+        && let Circuit::Array(a) = &**node.children_sl()[1]
         && !i.batch_index
         && i.batch_x {
-        let x = &node.nodes[0];
+        let x = &node.children_sl()[0];
         let idx = if i.index_dim < 0 {
             (i.index_dim % (a.ndim() as i64)) as usize
         } else {
