@@ -9,7 +9,7 @@ use crate::algebraic_rewrite::{get_removable_axes, remove_axes_weak};
 #[pyfunction]
 pub fn einsum_elim_removable_axes_weak(einsum: &Einsum) -> Option<Einsum> {
     let mut int_to_input: HashMap<u8, HashSet<usize>> = HashMap::default();
-    for (i, ints) in einsum.input_axes().enumerate() {
+    for (i, ints) in einsum.in_axes.iter().enumerate() {
         for j in ints {
             int_to_input
                 .entry(*j)
@@ -18,7 +18,7 @@ pub fn einsum_elim_removable_axes_weak(einsum: &Einsum) -> Option<Einsum> {
         }
     }
     let mut int_to_input_removable: HashMap<u8, _> = HashMap::default();
-    for (i, (node, ints)) in einsum.args.iter().enumerate() {
+    for (i, (node, ints)) in einsum.args().enumerate() {
         let removable = get_removable_axes(&node);
         for (axis, j) in ints.iter().enumerate() {
             if !removable.contains(&axis) {
@@ -56,8 +56,7 @@ pub fn einsum_elim_removable_axes_weak(einsum: &Einsum) -> Option<Einsum> {
 
     return Some(Einsum::new(
         einsum
-            .args
-            .iter()
+            .args()
             .enumerate()
             .map(|(i, (node, ints))| {
                 let removable_axes: HashSet<usize> = ints
@@ -79,7 +78,7 @@ pub fn einsum_elim_removable_axes_weak(einsum: &Einsum) -> Option<Einsum> {
                 // we have to reintersect with removeable because ints can
                 // correspond to multiple axes (e.g., ii,i->i)
                 let removable_axes: HashSet<usize> = removable_axes
-                    .intersection(&get_removable_axes(node))
+                    .intersection(&get_removable_axes(&node))
                     .cloned()
                     .collect();
 
@@ -87,8 +86,8 @@ pub fn einsum_elim_removable_axes_weak(einsum: &Einsum) -> Option<Einsum> {
                     (node.clone(), ints.clone())
                 } else {
                     (
-                        remove_axes_weak(node, &removable_axes).unwrap(),
-                        filter_out_idx(ints, &removable_axes).into_iter().collect(),
+                        remove_axes_weak(&node, &removable_axes).unwrap(),
+                        filter_out_idx(&ints, &removable_axes).into_iter().collect(),
                     )
                 }
             })
@@ -104,7 +103,7 @@ pub fn add_elim_removable_axes_weak(add: &Add) -> Option<Add> {
     let mut axis_to_inputs: HashMap<usize, HashSet<usize>> = HashMap::default();
     let mut axis_to_inputs_removable: HashMap<usize, _> = HashMap::default();
     for (i, v) in axis_map.iter().enumerate() {
-        let node = &add.nodes[i];
+        let node = &add.children_sl()[i];
         let removable = get_removable_axes(&node);
         for (oj, j) in v
             .iter()
