@@ -55,8 +55,7 @@ pub struct Dag {
 }
 impl Dag {
     pub fn get_outputs(&self) -> Vec<usize> {
-        let result =
-            rr_util::util::filter_to_idx(&self.parents.iter().collect(), &mut |y| y.is_empty());
+        let result = rr_util::util::filter_to_idx(self.parents.iter(), |y: &&Sv<_>| y.is_empty());
         result
     }
 
