@@ -885,7 +885,7 @@ impl IterativeMatcher {
                     .any(|x| matches!(x, ChainItemPerChild::Many(_)))
                 {
                     assert!(!finished);
-                    let num_children = circuit.children().count();
+                    let num_children = circuit.num_children();
                     let mut out = vec![BTreeSet::default(); num_children];
                     for item in new_items {
                         match item {
