@@ -10,7 +10,8 @@ pub mod sampler;
 pub mod transform;
 
 pub use iterative_matcher::{
-    IterateMatchResults, IterativeMatcher, IterativeMatcherData, IterativeMatcherRc,
+    new_traversal, restrict, restrict_sl, IterateMatchResults, IterativeMatcher,
+    IterativeMatcherData, IterativeMatcherRc,
 };
 pub use matcher::{Matcher, MatcherData, MatcherFromPy, MatcherFromPyBase, MatcherRc};
 pub use operations::{
