@@ -23,7 +23,7 @@ use rr_util::{
 use thiserror::Error;
 use uuid::uuid;
 
-use crate::{BoundAnyFound, IterativeMatcher, IterativeMatcherRc};
+use crate::{restrict, BoundAnyFound, IterativeMatcher, IterativeMatcherRc};
 
 make_single_many!(TypeTags, CircuitType, BTreeSet);
 make_single_many!(Strings, String, BTreeSet);
@@ -445,14 +445,14 @@ impl Matcher {
     #[staticmethod]
     pub fn match_any_child_found(finder: IterativeMatcherRc) -> Self {
         Self::match_any_found(
-            finder
-                .filter(
-                    false,
-                    Some(1),
-                    None,
-                    MatcherFromPyBase::Always(false).into(),
-                )
-                .into(),
+            restrict(
+                finder,
+                false,
+                Some(1),
+                None,
+                MatcherFromPyBase::Always(false).into(),
+            )
+            .into(),
         )
     }
 
