@@ -24,6 +24,12 @@ use crate::{
     IterateMatchResults, IterativeMatcherRc, Transform, TransformData, TransformRc,
 };
 
+#[derive(Debug, Clone)]
+pub(super) struct UpdateOutput {
+    pub(super) updated: CircuitRc,
+    pub(super) any_found: bool,
+}
+
 #[pyclass]
 #[derive(Debug, Clone)]
 pub struct Updater {
@@ -35,8 +41,10 @@ pub struct Updater {
     pub(super) cache_update: bool,
     #[pyo3(get, set)]
     pub(super) default_fancy_validate: bool,
+    #[pyo3(get, set)]
+    pub(super) default_assert_any_found: bool,
     pub(super) transform_cache: FastUnboundedCache<HashBytes, CircuitRc>,
-    pub(super) updated_cache: FastUnboundedCache<(HashBytes, IterativeMatcherRc), CircuitRc>,
+    pub(super) updated_cache: FastUnboundedCache<(HashBytes, IterativeMatcherRc), UpdateOutput>,
     pub(super) validation_getter: Getter,
 }
 
@@ -47,6 +55,7 @@ impl Default for Updater {
             cache_transform: true,
             cache_update: true,
             default_fancy_validate: false,
+            default_assert_any_found: false,
             transform_cache: FastUnboundedCache::default(),
             updated_cache: FastUnboundedCache::default(),
             validation_getter: Default::default(),
@@ -61,19 +70,22 @@ impl Updater {
         transform,
         cache_transform = Updater::default().cache_transform,
         cache_update = Updater::default().cache_update,
-        default_fancy_validate = Updater::default().default_fancy_validate
+        default_fancy_validate = Updater::default().default_fancy_validate,
+        default_assert_any_found = Updater::default().default_assert_any_found
     ))]
     pub fn new(
         transform: TransformRc,
         cache_transform: bool,
         cache_update: bool,
         default_fancy_validate: bool,
+        default_assert_any_found: bool,
     ) -> Self {
         Self {
             transform,
             cache_transform,
             cache_update,
             default_fancy_validate,
+            default_assert_any_found,
             ..Default::default()
         }
     }
@@ -84,8 +96,9 @@ impl Updater {
         circuit: CircuitRc,
         matcher: IterativeMatcherRc,
         fancy_validate: Option<bool>,
+        assert_any_found: Option<bool>,
     ) -> Result<CircuitRc> {
-        self.update(circuit, matcher, fancy_validate)
+        self.update(circuit, matcher, fancy_validate, assert_any_found)
     }
 
     pub fn update(
@@ -93,12 +106,24 @@ impl Updater {
         circuit: CircuitRc,
         matcher: IterativeMatcherRc,
         fancy_validate: Option<bool>,
+        assert_any_found: Option<bool>,
     ) -> Result<CircuitRc> {
         if fancy_validate.unwrap_or(self.default_fancy_validate) {
             self.validation_getter
                 .validate(circuit.clone(), matcher.clone())?;
         }
-        self.update_impl(circuit, matcher)
+        let out = self.update_impl(circuit.crc(), matcher.crc())?;
+        if assert_any_found.unwrap_or(self.default_assert_any_found) && !out.any_found {
+            bail!(
+                concat!(
+                    "No matches found during update\n",
+                    "matcher: {matcher:?}, circuit: {circuit:?}"
+                ),
+                matcher = matcher,
+                circuit = circuit
+            );
+        }
+        Ok(out.updated)
     }
 
     pub fn bind(&self, matcher: IterativeMatcherRc) -> BoundUpdater {
@@ -119,21 +144,28 @@ impl Updater {
         &mut self,
         circuit: CircuitRc,
         matcher: IterativeMatcherRc,
-    ) -> Result<CircuitRc> {
+    ) -> Result<UpdateOutput> {
         let IterateMatchResults { updated, found } = matcher.match_iterate(circuit.clone())?;
 
+        let mut any_found = false;
         let mut new_circuit =
             function_per_child(updated, matcher, circuit.clone(), |circuit, new_matcher| {
-                self_.update_impl(circuit, new_matcher)
+                let child_out = self_.update_impl(circuit, new_matcher)?;
+                any_found |= child_out.any_found;
+                Ok(child_out.updated)
             })?;
 
         if found {
+            any_found = true;
             if !matches!(self_.transform.data(), TransformData::Ident) {
                 new_circuit = self_.run_transform(new_circuit)?;
             }
         }
 
-        Ok(new_circuit)
+        Ok(UpdateOutput {
+            updated: new_circuit,
+            any_found,
+        })
     }
 
     #[apply(cached_method)]
@@ -167,17 +199,23 @@ impl BoundUpdater {
         _py: Python<'_>,
         circuit: CircuitRc,
         fancy_validate: Option<bool>,
+        assert_any_found: Option<bool>,
     ) -> Result<CircuitRc> {
-        self.update(circuit, fancy_validate)
+        self.update(circuit, fancy_validate, assert_any_found)
     }
 
     pub fn update(
         &mut self,
         circuit: CircuitRc,
         fancy_validate: Option<bool>,
+        assert_any_found: Option<bool>,
     ) -> Result<CircuitRc> {
-        self.updater
-            .update(circuit, self.matcher.clone(), fancy_validate)
+        self.updater.update(
+            circuit,
+            self.matcher.clone(),
+            fancy_validate,
+            assert_any_found,
+        )
     }
 }
 
@@ -628,10 +666,12 @@ pub struct Expander {
     pub ban_multiple_matches_on_node: bool,
     #[pyo3(set, get)]
     pub default_fancy_validate: bool,
+    #[pyo3(set, get)]
+    pub default_assert_any_found: bool,
     #[pyo3(get)]
     pub(super) suffix: Option<String>,
     pub(super) batch_cache:
-        FastUnboundedCache<(HashBytes, Vec<IterativeMatcherRc>, HashBytes), CircuitRc>,
+        FastUnboundedCache<(HashBytes, Vec<IterativeMatcherRc>, HashBytes), UpdateOutput>,
     pub(super) validation_getter: Getter,
 }
 
@@ -642,6 +682,7 @@ impl Default for Expander {
             matchers: Vec::new(),
             ban_multiple_matches_on_node: false,
             default_fancy_validate: false,
+            default_assert_any_found: false,
             suffix: None,
             batch_cache: FastUnboundedCache::default(),
             validation_getter: Default::default(),
@@ -656,12 +697,14 @@ impl Expander {
         *expanders,
         ban_multiple_matches_on_node = Expander::default().ban_multiple_matches_on_node,
         default_fancy_validate = Expander::default().default_fancy_validate,
+        default_assert_any_found = Expander::default().default_assert_any_found,
         suffix = None
     ))]
     pub fn new(
         expanders: Vec<(IterativeMatcherRc, TransformRc)>,
         ban_multiple_matches_on_node: bool,
         default_fancy_validate: bool,
+        default_assert_any_found: bool,
         suffix: Option<String>,
     ) -> Self {
         let (matchers, replacements) = expanders
@@ -673,6 +716,7 @@ impl Expander {
             matchers,
             ban_multiple_matches_on_node,
             default_fancy_validate,
+            default_assert_any_found,
             suffix,
             ..Default::default()
         }
@@ -683,18 +727,35 @@ impl Expander {
         _py: Python<'_>,
         circuit: CircuitRc,
         fancy_validate: Option<bool>,
+        assert_any_found: Option<bool>,
     ) -> Result<CircuitRc> {
-        self.batch(circuit, fancy_validate)
+        self.batch(circuit, fancy_validate, assert_any_found)
     }
 
-    pub fn batch(&mut self, circuit: CircuitRc, fancy_validate: Option<bool>) -> Result<CircuitRc> {
+    pub fn batch(
+        &mut self,
+        circuit: CircuitRc,
+        fancy_validate: Option<bool>,
+        assert_any_found: Option<bool>,
+    ) -> Result<CircuitRc> {
         if fancy_validate.unwrap_or(self.default_fancy_validate) {
             for m in &self.matchers {
                 self.validation_getter
                     .validate(circuit.clone(), m.clone())?;
             }
         }
-        self.batch_impl(circuit, self.matchers.clone(), &Default::default())
+        let out = self.batch_impl(circuit.crc(), self.matchers.clone(), &Default::default())?;
+        if assert_any_found.unwrap_or(self.default_assert_any_found) && !out.any_found {
+            bail!(
+                concat!(
+                    "No matches found during expand\n",
+                    "matchers: {matchers:?}, circuit: {circuit:?}"
+                ),
+                matchers = self.matchers,
+                circuit = circuit,
+            );
+        }
+        Ok(out.updated)
     }
 }
 
@@ -709,7 +770,7 @@ impl Expander {
         circuit: CircuitRc,
         matchers: Vec<IterativeMatcherRc>,
         extra_replacements: &ReplaceMapRc,
-    ) -> Result<CircuitRc> {
+    ) -> Result<UpdateOutput> {
         let results = matchers
             .iter()
             .map(|m| m.match_iterate(circuit.clone()))
@@ -725,14 +786,23 @@ impl Expander {
                 }
             }
 
-            return self_.replacements[idx].run(circuit);
+            return Ok(UpdateOutput {
+                updated: self_.replacements[idx].run(circuit)?,
+                any_found: true,
+            });
         }
         if let Some(replaced) = extra_replacements.get(&circuit) {
-            return Ok(replaced.clone());
+            return Ok(UpdateOutput {
+                updated: replaced.clone(),
+                any_found: false,
+            });
         }
 
         if results.iter().all(|x| all_finished(&x.updated)) && extra_replacements.is_empty() {
-            return Ok(circuit);
+            return Ok(UpdateOutput {
+                updated: circuit,
+                any_found: false,
+            });
         }
 
         let num_children = circuit.num_children();
@@ -745,19 +815,26 @@ impl Expander {
         let new_matchers_per = transpose(new_matchers, num_children);
         assert_eq!(new_matchers_per.len(), circuit.num_children());
 
+        let mut any_found = false;
         let new_children = circuit
             .children()
             .zip(extra_replacements.per_child(&circuit))
             .zip(new_matchers_per.clone())
-            .map(|((c, rep), new_matchers)| self_.batch_impl(c, new_matchers, &rep))
+            .map(|((c, rep), new_matchers)| {
+                let child_out = self_.batch_impl(c, new_matchers, &rep)?;
+                any_found |= child_out.any_found;
+                Ok(child_out.updated)
+            })
             .collect::<Result<_>>()?;
 
         let expanded = expand_node(circuit.clone(), &new_children, &mut |c, rep, child_idx| {
-            self_.batch_impl(
+            let out = self_.batch_impl(
                 c,
                 new_matchers_per[child_idx].clone(),
                 &extra_replacements.extend_into(rep),
-            )
+            )?;
+            any_found |= out.any_found;
+            Ok(out.updated)
         })?;
         let expanded = if expanded != circuit {
             expanded.add_suffix(self_.suffix.as_ref().map(|x| &**x))
@@ -765,7 +842,10 @@ impl Expander {
             expanded
         };
 
-        Ok(expanded)
+        Ok(UpdateOutput {
+            updated: expanded,
+            any_found,
+        })
     }
 
     pub fn suffix(&self) -> Option<&str> {
