@@ -244,9 +244,9 @@ pub fn fuse_modules_impl(
             (*items)
                 .to_owned()
                 .into_iter()
-                .zip(&m.nodes)
+                .zip(m.args())
                 .zip(m.spec.arg_specs.clone())
-                .zip(m.spec.batch_shapes(&m.nodes))
+                .zip(m.spec.batch_shapes(m.args_slice()))
                 .map(|(((node, orig_node), arg_spec), orig_batch_shape)| {
                     let missing_ndims = overall_mod_batch_shape.len() - orig_batch_shape.len();
                     let non_batch = orig_node.ndim() - orig_batch_shape.len();
@@ -341,8 +341,8 @@ impl NestedModuleItems {
         assert_eq!(modules.len(), new_nodes_flat.len());
         assert_eq!(modules.len(), new_nodes_base.len());
         for ((self_m, new), base) in modules.iter().zip(&new_nodes_flat).zip(&new_nodes_base) {
-            assert_eq!(self_m.nodes.len(), new.len());
-            assert_eq!(self_m.nodes.len(), base.len());
+            assert_eq!(self_m.num_args(), new.len());
+            assert_eq!(self_m.num_args(), base.len());
         }
 
         let flat = fuse_modules_impl(&modules, &new_nodes_flat)?;
@@ -770,30 +770,26 @@ impl ModulePusher {
             let mut new_iter = new_get.into_iter().zip(new_skip_module);
             let (spec_circuit_get, spec_circuit_skip_module) = new_iter.next().unwrap();
             let node_get_skip: Vec<_> = new_iter.collect();
-            assert_eq!(node_get_skip.len(), m.nodes.len());
+            assert_eq!(node_get_skip.len(), m.num_args());
 
             // intentionally don't use extra_replacements per child
             let new_nodes_flat_orig = m
-                .nodes
-                .clone()
-                .into_iter()
+                .args()
                 .zip(node_get_skip.clone())
                 .map(|(c, (get, skip))| {
-                    rec(self_, c, get, skip, items, extra_replacements).map(|x| x.0)
+                    rec(self_, c.clone(), get, skip, items, extra_replacements).map(|x| x.0)
                 })
                 .collect::<Result<Vec<_>>>()?;
 
             let (new_nodes_flat, new_nodes_base): (Vec<_>, _) = if push_or_get {
                 assert_eq!(extra_replacements.len(), 0);
-                let new_nodes_base = m.nodes.clone();
+                let new_nodes_base = m.args_cloned();
 
                 // we fully substitute in the 'get' case
                 // Argueably, this should push down for naming
 
                 (
-                    m.nodes
-                        .iter()
-                        .cloned()
+                    m.args()
                         .map(|circuit| {
                             let out = self_
                                 .build_module_flat_no_call(
@@ -811,9 +807,7 @@ impl ModulePusher {
                 )
             } else {
                 let new_nodes_base = m
-                    .nodes
-                    .clone()
-                    .into_iter()
+                    .args()
                     .zip(node_get_skip)
                     // don't use rec to avoid adding nodes to out_set
                     .map(|(c, (get, skip))| {
@@ -1195,7 +1189,7 @@ impl ModuleConstructUpdateBindings {
                 .rc();
             let name = m.info().name;
             Module::new(
-                m.nodes,
+                m.args_cloned(),
                 ModuleSpec {
                     circuit: new_spec_circuit,
                     arg_specs: m.spec.arg_specs,
@@ -1221,7 +1215,7 @@ impl ModuleConstructUpdateBindings {
         let m = if (self.is_flatten || count_mods == 1) && self.run_update_on_new_spec_circuits {
             if let Some(new_spec) = self.deep_update(m.spec.circuit.clone())? {
                 Module::try_new(
-                    m.nodes.clone(),
+                    m.args_cloned(),
                     ModuleSpec {
                         circuit: new_spec,
                         arg_specs: m.spec.arg_specs.clone(),
@@ -1476,8 +1470,7 @@ impl SymbolExtractor {
         let (circ, traversal_per_child) = if let Some(m) = circ.as_module() {
             let new_mod = if self_.conform_batch_if_needed {
                 let batch = m
-                    .nodes
-                    .iter()
+                    .args()
                     .zip(&m.spec.arg_specs)
                     .filter_map(|(node, arg_spec)| {
                         assert!(node.ndim() >= arg_spec.symbol.ndim());
@@ -1522,7 +1515,7 @@ impl SymbolExtractor {
                 })
                 .collect::<Result<Vec<_>>>()?;
             let (nodes, arg_specs, traversal_per_node): (Vec<_>, Vec<_>, Vec<_>) = multiunzip(iter);
-            let circ = if nodes.len() == m.nodes.len() {
+            let circ = if nodes.len() == m.num_args() {
                 m.crc()
             } else {
                 Module::nrc(
