@@ -1147,7 +1147,7 @@ where
 
     match new_item {
         NestIdxsItem::Single(_) => {
-            assert_eq!(flat.children().count(), 1);
+            assert_eq!(flat.num_children(), 1);
             Ok(flat.clone())
         }
         NestIdxsItem::Many(specs) => I::Full::nest_flat_strict_impl(flat, specs),
