@@ -2,7 +2,7 @@
 import functools
 import itertools
 import math
-from typing import List, Optional, Sequence, Tuple, Union
+from typing import Any, List, Optional, Sequence, Tuple, Union
 
 import torch
 import torch.nn.functional
@@ -14,8 +14,9 @@ def check_type(x, type_v):
 
 
 def get_tensor_shape(x):
-    check_type(x, torch.Tensor)
-    return list(x.size())
+    if not isinstance(x, torch.Tensor):
+        raise TypeError(f"expected type {repr(torch.Tensor)} but got {type(x)}")
+    return x.shape, x.device, x.dtype
 
 
 dtype_from_string = {
@@ -257,3 +258,31 @@ class OptimizingSymbolicSizeWarning(UserWarning):
 
 def random_i64():
     return int(torch.randint(-(2 ** 63), 2 ** 63 - 1, size=(), dtype=torch.long))
+
+
+def canon_index_tensor(x: torch.Tensor) -> Union[slice, torch.Tensor]:
+    if torch.all(torch.diff(x) == 1):
+        return slice(int(x[0]), int(x[0]) + len(x))
+    return x
+
+
+def index_union_rebase(indices, l: int, device: str):
+    mask = torch.zeros(l, dtype=torch.bool, device=device)
+    for idx in indices:
+        mask[idx] = True
+    nonzero = torch.nonzero(mask, as_tuple=True)[0]
+    rebased: List[Any] = []
+    if any([isinstance(x, torch.Tensor) for x in indices]):
+        place_in_new = torch.zeros(l, dtype=torch.int64, device=device)
+        place_in_new[nonzero] = torch.arange(len(nonzero), dtype=torch.int64, device=device)
+    for idx in indices:
+        if isinstance(idx, int):
+            rebased.append(torch.nonzero(nonzero == idx, as_tuple=True)[0].item())
+        elif isinstance(idx, slice):
+            # we know everything in the slice is in nonzero, so we can find start and take then next delta elements
+            start_idx = torch.nonzero(nonzero == idx.start, as_tuple=True)[0].item()
+            rebased.append(slice(start_idx, start_idx + (idx.stop - idx.start)))
+        else:
+            rebased.append(canon_index_tensor(place_in_new[idx]))
+    # print(f"index_union_rebase in: {indices} l: {l} out: {nonzero} {rebased}")
+    return (canon_index_tensor(nonzero), rebased)
