@@ -10,6 +10,9 @@ pub trait EqByBigHash {
     fn first_u64(&self) -> u64 {
         u64::from_le_bytes(self.first_u64_bytes())
     }
+    fn first_i64(&self) -> i64 {
+        i64::from_le_bytes(self.first_u64_bytes())
+    }
 }
 
 #[macro_export]
