@@ -11,7 +11,7 @@ use pyo3::{
 use rustc_hash::FxHashMap as HashMap;
 
 use crate::{
-    tensor_util::{Shape, TorchDeviceDtype, TorchDtype},
+    tensor_util::{Shape, TorchDevice, TorchDeviceDtype, TorchDtype},
     util::{EinsumAxes, HashBytes, HashBytesToPy},
 };
 pub struct PyUtils {
@@ -41,6 +41,8 @@ pub struct PyUtils {
     pub optimizing_symbolic_size_warning: PyObject,
     pub maybe_dtype_to_maybe_string: PyObject,
     pub random_i64: PyObject,
+    pub index_union_rebase: PyObject,
+    pub canon_index_tensor: PyObject,
 }
 
 /// misc python utilities
@@ -95,6 +97,8 @@ pub static PY_UTILS: GILLazyPy<PyUtils> = GILLazyPy::new_py(|py| {
         optimizing_symbolic_size_warning: get("OptimizingSymbolicSizeWarning"),
         maybe_dtype_to_maybe_string: get("maybe_dtype_to_maybe_string"),
         random_i64: get("random_i64"),
+        index_union_rebase: get("index_union_rebase"),
+        canon_index_tensor: get("canon_index_tensor"),
     }
 });
 
@@ -367,6 +371,7 @@ pub struct Tensor {
     tensor: PyObject,
     shape: Shape, /* cache shape so doesn't have to be recomputed on reconstruct etc (not uber efficient I think) */
     hash: Option<HashBytes>,
+    device_dtype: TorchDeviceDtype,
 }
 
 impl fmt::Debug for Tensor {
@@ -392,12 +397,16 @@ impl Eq for Tensor {}
 
 impl<'source> FromPyObject<'source> for Tensor {
     fn extract(tensor: &'source PyAny) -> PyResult<Self> {
-        let shape =
+        let (shape, device, dtype) =
             Python::with_gil(|py| PY_UTILS.get_tensor_shape.call1(py, (tensor,))?.extract(py))?;
 
         Ok(Self {
             tensor: tensor.into(),
             shape,
+            device_dtype: TorchDeviceDtype {
+                device: TorchDevice::from_python(device)?,
+                dtype: TorchDtype::from_python(dtype)?,
+            },
             hash: None,
         })
     }
@@ -432,6 +441,9 @@ impl Tensor {
     pub fn hash(&self) -> Option<&HashBytes> {
         self.hash.as_ref()
     }
+    pub fn device_dtype(&self) -> TorchDeviceDtype {
+        self.device_dtype
+    }
 
     pub fn hash_base16(&self) -> Option<String> {
         self.hash.as_ref().map(|x| base16::encode_lower(x))
@@ -467,6 +479,7 @@ impl Tensor {
             Ok(Self {
                 tensor: self.tensor.clone(),
                 shape: self.shape.clone(),
+                device_dtype: self.device_dtype,
                 hash: Python::with_gil(|py| {
                     Ok::<Option<HashBytes>, anyhow::Error>(
                         HASH_TENSOR
