@@ -30,11 +30,19 @@ use crate::{
     lru_cache::TensorCacheRrfs,
     py_types::{use_rust_comp, Tensor, PY_UTILS},
     pycall, python_error_exception, sv,
+    symbolic_size::SymbolicSizeProduct,
     tensor_db::{get_tensor_prefix, save_tensor},
     util::HashBytes,
 };
 pub type Shape = Sv<[usize; 4]>; // TODO: maybe this should be struct which preserves invariant that dim shapes >= 0?
 
+pub fn eq_or_symbolic(x: &[usize], y: &[usize]) -> bool {
+    x.len() == y.len()
+        && x.iter().zip(y).all(|(a, b)| {
+            SymbolicSizeProduct::has_symbolic(*a) || SymbolicSizeProduct::has_symbolic(*b) || a == b
+        })
+}
+
 #[crate::pyo3::pyfunction]
 #[pyo3(name = "broadcast_shapes", signature = (*shapes))]
 pub fn broadcast_shapes_py(shapes: Vec<Shape>) -> Result<PyShape> {
@@ -476,29 +484,76 @@ impl TensorAxisIndex {
         start: None,
         stop: None,
     });
-    /// untested
+
+    pub fn tensor_canon(tensor: &IndexTensor) -> Self {
+        pycall!(PY_UTILS.canon_index_tensor, (tensor,))
+    }
+
+    pub fn indices_contiguous_uslice(indices: &Vec<TensorAxisIndex>) -> Option<USlice> {
+        if let Some(mut containings) = indices
+            .iter()
+            .map(USlice::containing_uslice)
+            .collect::<Option<Vec<_>>>()
+        {
+            containings.sort_by(|a, b| a.start.cmp(&b.start));
+            let mut result = containings[0].clone();
+            for c in &containings[1..] {
+                if c.start > result.stop {
+                    return None;
+                }
+                result.stop = max(result.stop, c.stop)
+            }
+            return Some(result);
+        }
+        None
+    }
+
+    // assumes indices canonicalized
+    pub fn indices_union_and_rebased(
+        indices: &Vec<TensorAxisIndex>,
+        l: usize,
+        device_dtype: TorchDeviceDtype,
+    ) -> (TensorAxisIndex, Vec<TensorAxisIndex>) {
+        if let Some(contiguous) = TensorAxisIndex::indices_contiguous_uslice(indices) {
+            (
+                TensorAxisIndex::Slice(contiguous.into()),
+                indices
+                    .iter()
+                    .map(|x| x.shrink_base_uslice(&contiguous))
+                    .collect(),
+            )
+        } else {
+            pycall!(
+                PY_UTILS.index_union_rebase,
+                (indices.clone(), l, device_dtype.device.clone())
+            )
+        }
+    }
+
+    // assumes self canonicalized
     pub fn shrink_base_uslice(&self, uslice: &USlice) -> Self {
         match self {
             TensorAxisIndex::Single(single) => {
-                assert!(*single >= 0);
-                assert!((single + uslice.start as i64) < uslice.stop as i64);
                 TensorAxisIndex::Single(single - uslice.start as i64)
             }
-            TensorAxisIndex::Slice(slice) => {
-                assert!(
-                    slice.stop.is_some()
-                        && slice.stop.unwrap() >= 0
-                        && slice.start.unwrap_or(0) >= 0
-                );
-                let start = Some(slice.start.unwrap_or(0) - uslice.start as i64);
-                let stop = Some(slice.stop.unwrap() - uslice.start as i64);
-                TensorAxisIndex::Slice(Slice { start, stop })
-            }
+            TensorAxisIndex::Slice(slice) => TensorAxisIndex::Slice(Slice {
+                start: Some(slice.start.unwrap() - uslice.start as i64),
+                stop: Some(slice.stop.unwrap() - uslice.start as i64),
+            }),
             TensorAxisIndex::Tensor(_tensor) => {
                 unimplemented!();
             }
         }
     }
+
+    pub fn canonicalize(&self, l: usize) -> Self {
+        match self {
+            TensorAxisIndex::Single(i) => TensorAxisIndex::Single((*i + l as i64) % l as i64),
+            TensorAxisIndex::Tensor(t) => TensorAxisIndex::tensor_canon(t),
+            TensorAxisIndex::Slice(sl) => TensorAxisIndex::Slice(sl.canonicalize(l)),
+        }
+    }
+
     pub fn new_plain_slice(start: usize, stop: usize) -> Self {
         TensorAxisIndex::Slice(Slice {
             start: Some(start as i64),
@@ -507,10 +562,10 @@ impl TensorAxisIndex {
     }
 
     pub fn is_identity(&self, l: usize) -> bool {
-        if let TensorAxisIndex::Slice(slice) = self {
-            return slice.size(l) == l;
+        match self {
+            TensorAxisIndex::Slice(s) => s.is_identity(l),
+            _ => false, // dont bother to check tensor, if you want that canon first
         }
-        false
     }
 
     pub fn new_tensor_randint_seeded(
@@ -638,11 +693,7 @@ impl TensorIndex {
     }
 
     pub fn is_identity(&self, shape: &Shape) -> bool {
-        zip(&self.0, shape).all(|(idx, &l)| match idx {
-            TensorAxisIndex::Single(_i) => false,
-            TensorAxisIndex::Tensor(_t) => false, // dont bother to check tensor, if you want that canon first
-            TensorAxisIndex::Slice(sl) => sl.is_identity(l),
-        })
+        zip(&self.0, shape).all(|(idx, &l)| idx.is_identity(l))
     }
 
     pub fn validate(&self, shape: &Shape) -> Result<()> {
@@ -691,11 +742,7 @@ impl TensorIndex {
         self.validate(shape).expect("invalid tensor index");
         TensorIndex(
             zip(&self.0, shape)
-                .map(|(idx, &l)| match idx {
-                    TensorAxisIndex::Single(i) => TensorAxisIndex::Single((*i + l as i64) % l as i64),
-                    TensorAxisIndex::Tensor(t) => TensorAxisIndex::Tensor(t.clone()), // not bothering to canon tensor for now
-                    TensorAxisIndex::Slice(sl) => TensorAxisIndex::Slice(sl.canonicalize(l)),
-                })
+                .map(|(idx, &l)| idx.canonicalize(l))
                 .collect(),
         )
     }
@@ -1054,6 +1101,17 @@ macro_rules! string_enum{
                 }
             }
         }
+        impl TryFrom<&str> for $name{
+            type Error = Error;
+            fn try_from(value:&str)->Result<$name>{
+                match value{
+                    $(
+                        lit_or!($var $(,$lit)?)=>Ok($name::$var),
+                    )*
+                    _=>Err(anyhow!("invalid value for enum {}, {}", stringify!($name), value).into())
+                }
+            }
+        }
         impl IntoPy<PyObject> for $name {
             fn into_py(self, py: Python<'_>) -> PyObject {
                 String::from(&self).into_py(py)
@@ -1110,6 +1168,18 @@ impl TorchDtype {
     //         Self::int8 | Self::int16 | Self::int32 | Self::int64 | Self::uint8
     //     )
     // }
+    pub fn from_python(x: PyObject) -> Result<Self> {
+        Python::with_gil(|py| {
+            if !x
+                .as_ref(py)
+                .is_instance(&PY_UTILS.torch.getattr(py, "dtype")?.as_ref(py))?
+            {
+                bail!("not torch dtype")
+            }
+            let s: String = x.call_method1(py, "__str__", ())?.extract(py)?;
+            s[6..].to_owned().try_into()
+        })
+    }
 }
 
 string_enum! {
@@ -1117,7 +1187,6 @@ string_enum! {
     None,
     (
         Cpu="cpu",
-        Cuda="cuda",
         Cuda0="cuda:0",
         Cuda1="cuda:1",
         Cuda2="cuda:2",
@@ -1138,27 +1207,40 @@ string_enum! {
 }
 
 impl TorchDevice {
+    pub const CUDA: TorchDevice = TorchDevice::Cuda0;
+
     pub fn is_cuda(&self) -> bool {
-        matches!(
-            self,
-            Self::Cuda
-                | Self::Cuda0
-                | Self::Cuda1
-                | Self::Cuda2
-                | Self::Cuda3
-                | Self::Cuda4
-                | Self::Cuda5
-                | Self::Cuda6
-                | Self::Cuda7
-                | Self::Cuda8
-                | Self::Cuda9
-                | Self::Cuda10
-                | Self::Cuda11
-                | Self::Cuda12
-                | Self::Cuda13
-                | Self::Cuda14
-                | Self::Cuda15
-        )
+        matches!(self, |Self::Cuda0| Self::Cuda1
+            | Self::Cuda2
+            | Self::Cuda3
+            | Self::Cuda4
+            | Self::Cuda5
+            | Self::Cuda6
+            | Self::Cuda7
+            | Self::Cuda8
+            | Self::Cuda9
+            | Self::Cuda10
+            | Self::Cuda11
+            | Self::Cuda12
+            | Self::Cuda13
+            | Self::Cuda14
+            | Self::Cuda15)
+    }
+    pub fn from_python(x: PyObject) -> Result<Self> {
+        Python::with_gil(|py| {
+            if !x
+                .as_ref(py)
+                .is_instance(&PY_UTILS.torch.getattr(py, "device")?.as_ref(py))?
+            {
+                bail!("not torch device")
+            }
+            let s: String = x.getattr(py, "type")?.extract(py)?;
+            if &s == "cpu" {
+                return Ok(TorchDevice::Cpu);
+            }
+            let idx: usize = atr!(x, index);
+            ("cuda".to_owned() + ":" + &idx.to_string()).try_into()
+        })
     }
 }
 
@@ -1187,11 +1269,6 @@ impl TorchDeviceDtype {
     fn new(device: TorchDevice, dtype: TorchDtype) -> Self {
         Self { device, dtype }
     }
-    #[staticmethod]
-    #[pyo3(name = "from_tensor")]
-    pub fn from_tensor_py(tensor: Tensor) -> Self {
-        Self::from_tensor(&tensor)
-    }
 
     pub fn cast_tensor(&self, tensor: Tensor) -> Tensor {
         Python::with_gil(|py| {
@@ -1226,33 +1303,6 @@ impl TorchDeviceDtype {
 }
 
 impl TorchDeviceDtype {
-    pub fn from_tensor(tensor: &Tensor) -> Self {
-        Python::with_gil(|py| Self {
-            device: tensor
-                .tensor()
-                .getattr(py, "device")
-                .unwrap()
-                .getattr(py, "__str__")
-                .unwrap()
-                .call0(py)
-                .unwrap()
-                .extract(py)
-                .unwrap(),
-            dtype: tensor
-                .tensor()
-                .getattr(py, "dtype")
-                .unwrap()
-                .getattr(py, "__str__")
-                .unwrap()
-                .call0(py)
-                .unwrap()
-                .extract::<String>(py)
-                .unwrap()[6..]
-                .to_owned()
-                .try_into()
-                .unwrap(),
-        })
-    }
     pub fn size(&self) -> usize {
         match self.dtype {
             TorchDtype::float32 => 4,
@@ -1343,7 +1393,7 @@ impl TorchDeviceDtypeOp {
         result
     }
     pub fn cast_tensor(&self, tensor: Tensor) -> Tensor {
-        let concrete = self.overwrite(&TorchDeviceDtype::from_tensor(&tensor));
+        let concrete = self.overwrite(&tensor.device_dtype());
         concrete.cast_tensor(tensor)
     }
     pub fn hash(&self, hasher: &mut blake3::Hasher) {
