@@ -1,7 +1,6 @@
 from __future__ import annotations
 
-from enum import Enum
-from typing import Callable, Dict, List, Literal, NamedTuple, Optional, Set, Tuple, Union
+from typing import Dict, Literal, Optional, Union
 from warnings import warn
 
 import attrs
@@ -47,7 +46,7 @@ class ExperimentEvalSettings:
         num_samples=None,
         device_dtype: Union[rc.TorchDeviceDtypeOp, str] = rc.TorchDeviceDtypeOp("cuda:0"),
     ):
-        if run_on_all and (num_samples != None):
+        if run_on_all and (num_samples is not None):
             raise ValueError(f"num_samples should be None when run_on_all is True, but was {num_samples}")
         self.optimize = optimize
         self.run_on_all = run_on_all
@@ -122,7 +121,7 @@ class Experiment:
         self._dataset = dataset
         self._check = check
 
-        if not (self._check == True):
+        if self._check is not True:
             warn(f"You're using check = {self._check}, your correspondence may have issues that won't be caught!")
         if self._check:
             corr.check(self._base_circuit, circuit_treeified=False)
