@@ -299,7 +299,7 @@ class InterpNodeGeneralFunctionSpec(GeneralFunctionSpecBase):
 
     def get_shape_info(self, *shapes):
         # empty shape
-        return GeneralFunctionShapeInfo((), 0, [])
+        return GeneralFunctionShapeInfo((), 0, [False] * len(shapes))
 
 
 class InterpNode:
