@@ -25,7 +25,7 @@ from ._rust import Cumulant as rCumulant
 from ._rust import DiscreteVar as rDiscreteVar
 from ._rust import Einsum as rEinsum
 from ._rust import GeneralFunction as rGeneralFunction
-from ._rust import GeneralFunctionShapeInfo, GeneralFunctionSimpleSpec, GeneralFunctionSpecBase
+from ._rust import GeneralFunctionShapeInfo, GeneralFunctionSimpleSpec
 from ._rust import Index as rIndex
 from ._rust import Rearrange as rRearrange
 from ._rust import Scalar as rScalar
@@ -34,6 +34,7 @@ from ._rust import Schedule as rSchedule
 from ._rust import StoredCumulantVar as rStoredCumulantVar
 from ._rust import Tag as rTag
 from ._rust import TorchDeviceDtypeOp, scatter_to_concat
+from .general_function_spec_base import GeneralFunctionSpecBase
 
 MYPY = False
 
