@@ -184,7 +184,7 @@ batchnorm_str = f"""
 batchnorm_circs = add_new_circs(batchnorm_str, {})
 batchnorm = ModuleLibraryRet.mk(batchnorm_circs, "bn")
 
-not_mask_str = f"""
+not_mask_str = """
 'not_mask' Add
   'one' [] Scalar 1.
   'not_mask.neg_mask' Einsum ,->
@@ -599,7 +599,7 @@ class TransformerBlockParams:
           """
             circs = add_new_circs(block_str, circs)
             circs = get_norm_call("p", circs, mod="b.add").circs
-            block_str = f"""
+            block_str = """
           'b.resid.p' Add
             'b.p' Module
               'p.norm_call'
@@ -772,11 +772,11 @@ class TransformerParams:
         # keeps right ordering
         circs = dict(x for b in blocks for x in b.circs.items())
 
-        out: Circuit = Symbol((SEQ, HIDDEN), UUID("ece2bb5d-c6d6-4b7a-93a5-32b5ac264888"), f"t.input")
+        out: Circuit = Symbol((SEQ, HIDDEN), UUID("ece2bb5d-c6d6-4b7a-93a5-32b5ac264888"), "t.input")
         assert "t.input" not in circs
         circs["t.input"] = out
         for i, block in enumerate(blocks):
-            out = module_new_bind(block.body, (f"b.input", out), name=f"b{i}.call")
+            out = module_new_bind(block.body, ("b.input", out), name=f"b{i}.call")
             assert out.shape == (SEQ, HIDDEN)
             circs[out.name] = out
 
@@ -930,10 +930,10 @@ class TransformerInfo:
                 (torch.arange(seq_len)[:, None] >= torch.arange(seq_len)[None, :]).to(
                     device=inp_tok_embeds.device, dtype=inp_tok_embeds.torch_dtype or torch.float32
                 ),
-                f"t.a.c.causal_mask",  # prefix doesn't apply to constants
+                "t.a.c.causal_mask",  # prefix doesn't apply to constants
             )
         else:
-            mask = Scalar(1.0, shape=(seq_len, seq_len), name=f"t.a.c.full_mask")
+            mask = Scalar(1.0, shape=(seq_len, seq_len), name="t.a.c.full_mask")
 
         if inp_mask is not None:
 
@@ -1000,7 +1000,6 @@ def load_transformer_model_string(
     s = s.strip("\n")
     assert s.startswith("# info:")
     info_s, _, _ = s.partition("\n")
-    import interp
 
     info = cattrs.structure(json.loads(info_s.removeprefix("# info:")), TransformerInfo)
 
