@@ -1,20 +1,12 @@
-import itertools
-import math
 from datetime import timedelta
-from typing import Sequence
 from uuid import UUID
 
-import einops
 import hypothesis
 import hypothesis.extra.numpy as st_np
 import pytest
 import torch
 from torch import tensor
-from torch.testing import assert_close
 
-from interp.circuit.algebric_rewrite import MUL_REST, rearrange_muls
-from interp.circuit.circuit import Circuit, MemoizedFn
-from interp.circuit.circuit_utils import apply_to_mean_fn, bfs_topological_sort, cast_circuit, evaluate_fn
 from interp.circuit.computational_node import (
     Add,
     Concat,
@@ -22,50 +14,26 @@ from interp.circuit.computational_node import (
     GeneralFunction,
     Index,
     UnaryRearrange,
-    log_softmax_fn,
-    sigmoid_fn,
     softmax_fn,
     softmax_jacobian,
 )
-from interp.circuit.constant import ArrayConstant, Constant, FloatConstant, One, Zero
+from interp.circuit.constant import ArrayConstant, FloatConstant, One, Zero
 from interp.circuit.cum_algo import eps_attrib
-from interp.circuit.cumulant import CircuitMultiset, Cumulant
-from interp.circuit.cumulant_cnf import circuit_and, circuit_not, circuit_or, cnf_circuit_gaussian
-from interp.circuit.get_update_node import replace_circuit
-from interp.circuit.partitioning import partition
-from interp.circuit.sampling_estimation import (
-    estimate_cumulant_val_with_samples,
-    get_make_default_eval_many,
-    get_sampled_circuit_vals,
-)
-from interp.circuit.test_circuit import (
-    make_discrete_var,
-    make_discrete_var_test_circuits,
-    make_test_circuits,
-    random_positive_definite_tensor,
-)
+from interp.circuit.cumulant import Cumulant
+from interp.circuit.test_circuit import make_discrete_var
 from interp.circuit.testing.topdown_circuit import CircuitProbs as CP
 from interp.circuit.testing.topdown_circuit import st_Circuit
-from interp.circuit.testing.utils import target_diff
-from interp.circuit.var import AutoTag, DiscreteVar, StoredCumulantVar, Var
-from rust_circuit import Add as rAdd
+from interp.circuit.var import AutoTag, DiscreteVar
 from rust_circuit import Circuit as rCircuit
-from rust_circuit import Concat as rConcat
 from rust_circuit import Cumulant as rCumulant
 from rust_circuit import DiscreteVar as rDiscreteVar
 from rust_circuit import Einsum as rEinsum
 from rust_circuit import GeneralFunction as rGeneralFunction
-from rust_circuit import Index as rIndex
-from rust_circuit import PrintOptions
-from rust_circuit import Rearrange as rRearrange
 from rust_circuit import RunDiscreteVarAllSpec, Sampler
 from rust_circuit import StoredCumulantVar as rStoredCumulantVar
-from rust_circuit import Tag as rTag
-from rust_circuit import TorchDeviceDtypeOp
-from rust_circuit import Var as rVar
 from rust_circuit import rewrite_cum_to_circuit_of_cum
 from rust_circuit.cum_algo import eps_attrib as r_eps_attrib
-from rust_circuit.interop_rust import py_to_rust, rust_get_f64_evaluator
+from rust_circuit.interop_rust import py_to_rust
 
 
 @hypothesis.example(  # Two DiscreteVars
