@@ -62,24 +62,12 @@ def test_cast_cuda():
 
 def test_custom_generalfunction_device_dtype():
     class MyCustomSigmoid(rc.GeneralFunctionSpecBase):
-        @property
-        def name(self) -> str:
-            return "my_sigmoid"
-
-        def compute_hash_bytes(self) -> bytes:
-            return uuid.UUID("36a3ef34-a3cb-49a5-8cae-89f29f269d78").bytes
-
         def function(self, x: torch.Tensor) -> torch.Tensor:  # type: ignore[override]
             return torch.sigmoid(x)
 
         def get_shape_info(self, *shapes: rc.Shape) -> rc.GeneralFunctionShapeInfo:
             return rc.get_shape_info_simple(shapes)
 
-        @classmethod
-        def new(cls, circ: rc.Circuit, name: Optional[str] = None):
-            """convenience function"""
-            return rc.GeneralFunction(circ, spec=cls(), name=name)
-
         # optionally you can implement `get_device_dtype_override`, which validates input device/dtypes and optionally returns the output device/dtype
         def get_device_dtype_override(self, *device_dtypes: rc.TorchDeviceDtypeOp) -> Optional[rc.TorchDeviceDtypeOp]:
             print("DOING OVERRIDE")
