@@ -174,7 +174,7 @@ def test_basic_matching():
 
     assert Matcher.regex("^my.name.has_dot$")(Scalar(2.0, shape=(2, 3), name="my.name.has_dot"))
     assert not Matcher.regex("^my.name...s_dot$")(Scalar(2.0, shape=(2, 3), name="my.name.has_dot"))
-    assert Matcher.regex(rf"^my.name.\.\.s_dot$")(Scalar(2.0, shape=(2, 3), name="my.name.has_dot"))
+    assert Matcher.regex(r"^my.name.\.\.s_dot$")(Scalar(2.0, shape=(2, 3), name="my.name.has_dot"))
     assert Matcher.regex(r"^\.*$")(c)
     assert not Matcher.regex(r"^\.*$")(c.rename(None))
 
