@@ -1,5 +1,3 @@
-import itertools
-import math
 import random
 from typing import Iterable, Optional, Tuple
 
@@ -8,7 +6,6 @@ import pytest
 import torch
 
 import rust_circuit as rc
-from interp.circuit.computational_node import num_segments
 from interp.tools.indexer import TORCH_INDEXER as I
 from rust_circuit import (
     Add,
@@ -84,7 +81,6 @@ def test_correct_cumulant_estimate(seed=283838):
     probs_and_group = Array(probs)
 
     vars_weighted = [DiscreteVar(Array(v), probs_and_group) for v in vals]
-    val_weighted = (probs ** 2).sum()
 
     def estimate_and_comp(circuit, actual):
         # todo: add is_full to rust
