@@ -34,20 +34,15 @@ use rr_util::{
 pub struct ExceptionWithRustContext {
     #[pyo3(get)]
     exc: PyObject,
-    #[pyo3(get)]
-    context: Vec<String>,
+    context: anyhow::Error,
 }
 
 #[pymethods]
 impl ExceptionWithRustContext {
     fn __str__(&self) -> PyResult<String> {
-        Python::with_gil(|py| {
-            let x: &str = &self.exc.as_ref(py).str()?.to_string_lossy();
-            Ok(if self.context.len() == 0 {
-                x.to_owned()
-            } else {
-                format!("{}\n{}\n  (this is an ExceptionWithRustContext, access e.exc or upgrade to 3.11 to get the underlying exception)", x, self.context.join("\n"))
-            })
+        Python::with_gil(|_py| {
+            let ctx = &self.context;
+            Ok(format!("{ctx:?}\n(this is an ExceptionWithRustContext, access e.exc or upgrade to 3.11 to get the underlying exception)\n"))
         })
     }
 }
@@ -61,8 +56,8 @@ macro_rules! setup_errors {
                 }
             )*
 
-            pyo3::anyhow::anyhow_to_py_err(&err, |py, e, msgs|
-                PyErr::from_value(Py::new(py, ExceptionWithRustContext { exc: e.into_value(py).into(), context: msgs }).unwrap().as_ref(py)) )
+            pyo3::anyhow::anyhow_to_py_err(err, |py, e, ctx|
+                PyErr::from_value(Py::new(py, ExceptionWithRustContext { exc: e.into_value(py).into(), context: ctx }).unwrap().as_ref(py)) )
         }
         pub fn register_exceptions(py: Python<'_>, m: &PyModule) -> PyResult<()> {
             $(
